"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TSCountry = void 0;
const commons_1 = require("./commons");
const tsdefaults_1 = require("./tsdefaults");
const tsobject_1 = require("./tsobject");
const types_1 = require("./types");
const countries_json_1 = __importDefault(require("./countries.json"));
/**
 *  WARNING ABOUT countries.json
 *  - Ireland has no localeLanguage : our EULocale is used
 *    instead en language to match a better date format conformance
 *  - There's no US country there since the managed countries are
 *    maint to be the european ones. It may evolve in time.
 */
class TSCountry extends tsobject_1.TSRootObject {
    constructor(infos, locales) {
        super();
        this.alpha2Code = infos.alpha2Code;
        this.alpha3Code = infos.alpha3Code;
        this.names = infos.names;
        this.domains = infos.domains;
        this.EEC = infos.EEC;
        this.spokenLanguages = infos.spokenLanguages;
        this.dialCode = infos.dialCode;
        this.currency = infos.currency;
        this.locales = Object.assign({}, locales); // we take a copy here
    }
    static loadCountries(localesMap, managedLanguages) {
        if (!(0, commons_1.$ok)(TSCountry.__countriesMap)) {
            TSCountry.__countriesMap = new Map();
            TSCountry.__countries = [];
            countries_json_1.default.forEach(infos => {
                var _a;
                let loc = (0, commons_1.$ok)(infos.localeLanguage) ? localesMap.get(infos.localeLanguage) : TSCountry.__EULocale;
                if (!(0, commons_1.$ok)(loc)) {
                    loc = TSCountry.__EULocale;
                }
                const c = new TSCountry(infos, loc);
                c.locales.startingWeekDay = infos.startingWeekDay;
                TSCountry.__countries.push(c);
                TSCountry.__countriesMap.set(c.alpha2Code, c).set(c.alpha3Code, c);
                managedLanguages.forEach(l => TSCountry.__countriesMap.set((0, commons_1.$ascii)(c.names[l].toUpperCase()), c));
                (_a = infos.aliases) === null || _a === void 0 ? void 0 : _a.forEach(a => TSCountry.__countriesMap.set((0, commons_1.$ascii)(a.toUpperCase()), c));
            });
        }
    }
    static country(c) {
        c = (0, commons_1.$trim)(c);
        if (c.length) {
            if (!(0, commons_1.$ok)(TSCountry.__countriesMap)) {
                tsdefaults_1.TSDefaults.defaults(); /* this initializes everything */
            }
            const ret = TSCountry.__countriesMap.get((0, commons_1.$ascii)(c.toUpperCase()));
            if ((0, commons_1.$ok)(ret)) {
                return ret;
            }
        }
        return null;
    }
    static countries() {
        if (!(0, commons_1.$ok)(TSCountry.__countriesMap)) {
            tsdefaults_1.TSDefaults.defaults(); /* this initializes everything */
        }
        return [...TSCountry.__countries];
    }
    // WARNING: we return the name in the current language
    get name() { return this.names[(0, tsdefaults_1.$language)()]; }
    // WARNING: this is not the spoken language, but the language we use
    // as locales for this country. Idem for the language name.
    get language() { return this.locales.language; }
    get languageName() { return this.locales.names[(0, tsdefaults_1.$language)()]; }
    translatedName(lang) {
        const ret = this.names[(0, commons_1.$ok)(lang) ? lang : (0, tsdefaults_1.$language)()];
        return (0, commons_1.$length)(ret) ? ret : null;
    }
    toString() { return this.alpha2Code; }
    toJSON() { return this.alpha2Code; }
}
exports.TSCountry = TSCountry;
TSCountry.__EULocale = {
    language: types_1.Languages.en,
    names: { fr: "anglais", en: "english", de: "englisch", it: "inglese", es: "inglés", pt: "inglês", nl: "Engels" },
    months: ["January", "February", "March", "April", "May", "June", "July", "August", "Septembre", "Octobre", "Novembre", "Decembre"],
    shortMonths: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"],
    days: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"],
    shortDays: ["Sun.", "Mon.", "Tue.", "Wed.", "Thu.", "Fri.", "Sat."],
    startingWeekDay: 1,
    dateTimeFormat: "%d/%m/%Y %H:%M:%S",
    dateFormat: "%d/%m/%Y",
    shortDateFormat: "%d/%m/%y",
    shortDateTimeFormat: "%d/%m/%y %H:%M:%S",
    timeFormat: "%H:%M:%S",
    partialTimeFormat: "%H:%M",
    ampm: ['AM', 'PM']
};
//# sourceMappingURL=tscountry.js.map