"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.$weekOfYear = exports.$dayOfWeekFromTimestamp = exports.$dayOfYear = exports.$timestampWithoutTime = exports.$hourFromTimestamp = exports.$minuteFromTimestamp = exports.$secondFromTimestamp = exports.$timestamp = exports.$timeisvalid = exports.$dayisvalid = exports.$isleap = exports.TSMaxTimeStamp = exports.TSSecsFrom19700101To20010101 = exports.TSSecsFrom00010101To20010101 = exports.TSDaysFrom00010101To20010101 = exports.TSDaysFrom00000229To20010101 = exports.TSWeek = exports.TSDay = exports.TSHour = exports.TSMinute = exports.TSDate = void 0;
/**
 * TSDate is a timestamp (commencing at 01/01/2001) date class
 * meant to manipulate dates without any TZ (even if creation
 * with GMT representation exist and toLocaleString() method
 * offers representation in current TZ).
 *
 * TSDate instances are immutable which means you cannot change
 * their content after creation
 */
const tsdatecomp_1 = require("./tsdatecomp");
const commons_1 = require("./commons");
const compare_1 = require("./compare");
const types_1 = require("./types");
class TSDate {
    constructor() {
        let n = arguments.length;
        if (n >= 3) {
            if (!$dayisvalid(arguments[0], arguments[1], arguments[2])) {
                throw `Bad TSDate(${arguments[0]}, ${arguments[1]}, ${arguments[2]}) day arguments`;
            }
            if (n !== 3 && n !== 6) {
                throw `Impossible to initialize a new TSDate() with ${n} arguments`;
            }
            if (n === 6) {
                if (!$timeisvalid(arguments[3], arguments[4], arguments[5])) {
                    throw `Bad TSDate(,,,${arguments[3]}, ${arguments[4]}, ${arguments[5]}) time arguments`;
                }
                this._timestamp = $timestamp(arguments[0], arguments[1], arguments[2], arguments[3], arguments[4], arguments[5]);
            }
            else {
                this._timestamp = $timestamp(arguments[0], arguments[1], arguments[2], 0, 0, 0);
            }
        }
        else if (n === 2) {
            throw "Impossible to initialize a new TSDate() with 2 arguments";
        }
        else { // n === 1 || n === 0
            let t = arguments[0]; // undefined if n === 0
            if ((0, commons_1.$isnumber)(t)) {
                t = t | 0; // we trash all info after the second
                this._timestamp = Math.min(Math.max(-exports.TSSecsFrom00010101To20010101, t), exports.TSMaxTimeStamp);
            }
            else if (t instanceof TSDate) {
                this._timestamp = t.timestamp;
            }
            else {
                let comps = null;
                if ((0, commons_1.$isstring)(t)) {
                    if (t === TSDate.FUTURE) {
                        this._timestamp = exports.TSMaxTimeStamp;
                        return;
                    }
                    else if (t === TSDate.PAST) {
                        this._timestamp = -exports.TSSecsFrom00010101To20010101;
                        return;
                    }
                    else {
                        comps = (0, tsdatecomp_1.$isostring2components)(t);
                    }
                }
                else if (!(0, commons_1.$ok)(t) || t instanceof Date) {
                    comps = (0, tsdatecomp_1.$components)(t);
                }
                if (!(0, commons_1.$ok)(comps)) {
                    throw "Bad TSDate constructor parameters";
                }
                this._timestamp = (0, tsdatecomp_1.$components2timestamp)(comps);
            }
        }
    }
    static past() { return new TSDate(TSDate.PAST); }
    static future() { return new TSDate(TSDate.FUTURE); }
    static zulu() { return new TSDate((0, commons_1.$div)(Date.now(), 1000) - exports.TSSecsFrom19700101To20010101); }
    // this specific method takes a Date representation 
    // in Zulu timezone.
    static fromZulu(s) {
        if (!(0, commons_1.$ok)(s)) {
            return null;
        }
        return this.fromComponents({
            year: s.getUTCFullYear(),
            month: (s.getUTCMonth() + 1),
            day: s.getUTCDate(),
            hour: s.getUTCHours(),
            minute: s.getUTCMinutes(),
            second: s.getUTCSeconds(),
            dayOfWeek: s.getUTCDay()
        });
    }
    static fromEpoch(timestamp) {
        if (!(0, commons_1.$ok)(timestamp)) {
            return null;
        }
        return new TSDate(timestamp - exports.TSSecsFrom19700101To20010101);
    }
    static from(date) {
        if (!(0, commons_1.$ok)(date)) {
            return null;
        }
        if ((0, commons_1.$isnumber)(date)) {
            return this.fromTimeStamp(date);
        }
        if (date instanceof Date) {
            return this.fromDate(date);
        }
        return this.fromIsoString(date);
    }
    static fromTimeStamp(d) {
        return (0, commons_1.$ok)(d) ? new TSDate(d) : null;
    }
    static fromDate(d) {
        return (0, commons_1.$ok)(d) ? new TSDate(d) : null;
    }
    // usage TSDate.fromComponents(myComponents)
    // if you only want to set a day, that's up to you to put 0 in hour, minute and second fields
    static fromComponents(comp) {
        if (!(0, tsdatecomp_1.$componentsarevalid)(comp)) {
            return null;
        }
        const c = comp;
        // we use the timestamp constructor because we know that our date is valid
        return new TSDate($timestamp(c.year, c.month, c.day, c.hour, c.minute, c.second));
    }
    static fromIsoString(s) {
        if (!(0, commons_1.$ok)(s)) {
            return null;
        }
        return this.fromComponents((0, tsdatecomp_1.$parsedatetime)(s, tsdatecomp_1.TSDateForm.ISO8601));
    }
    // usage TSDate.fromString(aString[, parsing form])
    static fromString(s, form = tsdatecomp_1.TSDateForm.Standard) {
        return this.fromComponents((0, tsdatecomp_1.$parsedatetime)(s, form));
    }
    // usage TSDate.fromDateString(aString[, parsing form]).
    // this function only parse day date string (no time should be included in the string)
    static fromDateString(s, form = tsdatecomp_1.TSDateForm.Standard) {
        return this.fromComponents((0, tsdatecomp_1.$parsedate)(s, form));
    }
    // ================= instance methods ========================
    clone() { return new TSDate(this._timestamp); }
    get timestamp() { return this._timestamp; }
    isLeap() { return $isleap((0, tsdatecomp_1.$components)(this._timestamp).year); }
    dateWithoutTime() { return new TSDate($timestampWithoutTime(this._timestamp)); }
    timeSinceDate(d) { return this._timestamp - d.timestamp; }
    get year() { return (0, tsdatecomp_1.$components)(this._timestamp).year; }
    get month() { return (0, tsdatecomp_1.$components)(this._timestamp).month; }
    get day() { return (0, tsdatecomp_1.$components)(this._timestamp).day; }
    get hour() { return $hourFromTimestamp(this._timestamp); }
    get minute() { return $minuteFromTimestamp(this._timestamp); }
    get second() { return $secondFromTimestamp(this._timestamp); }
    week(offset = 0) { return $weekOfYear(this._timestamp, offset); }
    dayOfYear() { return $dayOfYear(this._timestamp); }
    dayOfWeek(offset = 0) { return $dayOfWeekFromTimestamp(this._timestamp, offset); }
    lastDayOfMonth() { let c = (0, tsdatecomp_1.$components)(this._timestamp); return _lastDayOfMonth(c.year, c.month); }
    dateByAdding(years, months = 0, days = 0, hours = 0, minutes = 0, seconds = 0) {
        if (!(0, commons_1.$isint)(years) || !(0, commons_1.$isint)(months) || !(0, commons_1.$isint)(days) || !(0, commons_1.$isint)(hours) || !(0, commons_1.$isint)(minutes) || !(0, commons_1.$isint)(seconds)) {
            throw "TSDate.dateByAdding() non integer arguments";
        }
        let ts = this._timestamp + days * exports.TSDay + hours * exports.TSHour + minutes * exports.TSMinute + seconds;
        if (!years && !months) {
            return new TSDate(ts);
        }
        let comp = (0, tsdatecomp_1.$components)(ts);
        const yearsInMonths = (0, commons_1.$div)(Math.abs(months), 12);
        years += months > 0 ? yearsInMonths : -yearsInMonths;
        months += yearsInMonths * (months > 0 ? -12 : 12);
        years += comp.year;
        months += comp.month;
        if (months > 12) {
            years++;
            months -= 12;
        }
        if (months < 1) {
            years--;
            months += 12;
        }
        if (years < 1) {
            return TSDate.past();
        }
        if (comp.day > TSDaysInMonth[months]) {
            comp.day = (months == 2 && $isleap(years) ? 29 : TSDaysInMonth[months]);
        }
        return new TSDate($timestamp(years, months, comp.day, comp.hour, comp.minute, comp.second));
    }
    dateByAddingWeeks(weeks) { return new TSDate(this._timestamp + weeks * exports.TSWeek); }
    dateByAddingTime(time) { return new TSDate(this._timestamp + time); }
    firstDateOfYear() { let c = (0, tsdatecomp_1.$components)(this._timestamp); return new TSDate(c.year, 1, 1); }
    lastDateOfYear() { let c = (0, tsdatecomp_1.$components)(this._timestamp); return new TSDate(c.year, 12, 31); }
    firstDateOfMonth() { let c = (0, tsdatecomp_1.$components)(this._timestamp); return new TSDate(c.year, c.month, 1); }
    lastDateOfMonth() { let c = (0, tsdatecomp_1.$components)(this._timestamp); return new TSDate(c.year, c.month, _lastDayOfMonth(c.year, c.month)); }
    firstDateOfWeek(offset = 0) {
        const ts = $timestampWithoutTime(this._timestamp);
        const dayOfWeek = $dayOfWeekFromTimestamp(ts, offset);
        return new TSDate(ts - dayOfWeek * exports.TSDay);
    }
    ;
    daysSinceDate(aDate) {
        return (0, commons_1.$div)($timestampWithoutTime(this._timestamp) - $timestampWithoutTime(aDate._timestamp), exports.TSDay);
    }
    toNumber() { return this._timestamp; }
    toEpoch() { return this._timestamp + exports.TSSecsFrom19700101To20010101; }
    toComponents() { return (0, tsdatecomp_1.$components)(this._timestamp); }
    toEpochTimestamp() { return this._timestamp + exports.TSSecsFrom19700101To20010101; } // DEPRECATED, use toEpoch()
    toDate() {
        return (0, tsdatecomp_1.$components2date)((0, tsdatecomp_1.$components)(this._timestamp));
    }
    toIsoString(format = tsdatecomp_1.TSDateForm.ISO8601) {
        return (0, tsdatecomp_1.$components2string)((0, tsdatecomp_1.$components)(this._timestamp), format);
    }
    // if we use this method it may be because the caller thinks he used a Date object
    toISOString() {
        console.log('toISOString() method should not be used on TSDate instances. Use toIsoString() instead');
        return this.toIsoString();
    }
    // ============ TSObject conformance =============== 
    get isa() { return this.constructor; }
    get className() { return this.constructor.name; }
    compare(other) {
        if (this === other) {
            return types_1.Same;
        }
        if (other instanceof TSDate) {
            return (0, compare_1.$numcompare)(this._timestamp, other.timestamp);
        }
        return undefined;
    }
    isEqual(other) { return this === other || (other instanceof TSDate && other.timestamp === this._timestamp); }
    toString(format, locale) {
        if (!(0, commons_1.$ok)(format) && !(0, commons_1.$ok)(locale)) {
            return this.toIsoString();
        }
        return this._toString(false, format, locale);
    }
    toLocaleString(format, locale) {
        return this._toString(true, format, locale);
    }
    toJSON() { return this.toIsoString(); }
    toArray() { return [this]; }
    // ============ Private methods =============== 
    _toString(localTime, format, locale) {
        const offset = localTime ? -(this.toDate()).getTimezoneOffset() * exports.TSMinute : 0;
        if ((0, commons_1.$isstring)(format)) {
            const ret = (0, tsdatecomp_1.$components2stringformat)((0, tsdatecomp_1.$components)(this._timestamp + offset), format, locale);
            return (0, commons_1.$ok)(ret) ? ret : '';
        }
        if (!(0, commons_1.$ok)(format)) {
            format = tsdatecomp_1.TSDateForm.Standard;
        }
        return (0, tsdatecomp_1.$components2string)((0, tsdatecomp_1.$components)(this._timestamp + offset), format);
    }
}
exports.TSDate = TSDate;
TSDate.FUTURE = 'future';
TSDate.PAST = 'past';
/***************************************************************************************************************
 * PUBLIC FUNCTIONS AND CONSTANTS
 ***************************************************************************************************************/
exports.TSMinute = 60;
exports.TSHour = 3600;
exports.TSDay = 86400;
exports.TSWeek = 604800;
exports.TSDaysFrom00000229To20010101 = 730792;
exports.TSDaysFrom00010101To20010101 = 730485;
exports.TSSecsFrom00010101To20010101 = 63113904000;
exports.TSSecsFrom19700101To20010101 = 978307200; // this one is exported for conversion from EPOCH to TSDate
exports.TSMaxTimeStamp = 8639021692800; // corresponds to JS 100 000 000 days from EPOCH in the future
function $isleap(y) {
    return !(0, commons_1.$isint)(y) || y % 4 ? false : (y % 100 ? (y > 7 ? true : false) : (y % 400 || y < 1600 ? false : true));
}
exports.$isleap = $isleap;
function $dayisvalid(year, month, day) {
    if (!(0, commons_1.$isint)(day) || !(0, commons_1.$isint)(month) || !(0, commons_1.$isint)(year) || day < 1 || day > 31 || month < 1 || month > 12) {
        return false;
    }
    if (day > TSDaysInMonth[month]) {
        return (month === 2 && day === 29 && $isleap(year)) ? true : false;
    }
    return true;
}
exports.$dayisvalid = $dayisvalid;
function $timeisvalid(hour, minute, second) {
    return ((0, commons_1.$isint)(hour) && (0, commons_1.$isint)(minute) && (0, commons_1.$isnumber)(second) && hour >= 0 && hour < 24 && minute >= 0 && minute < 60 && second >= 0 && second < 60);
}
exports.$timeisvalid = $timeisvalid;
function $timestamp(year, month, day, hours = 0, minutes = 0, seconds = 0) {
    if (!$dayisvalid(year, month, day)) {
        throw `Bad $timestamp(${year}, ${month}, ${day}) day arguments`;
    }
    if (!$timeisvalid(hours, minutes, seconds)) {
        throw `Bad  $timestamp(,,,${hours}, ${minutes}, ${seconds}) time arguments`;
    }
    if (month < 3) {
        month += 12;
        year--;
    }
    let leaps = Math.floor(year / 4) - Math.floor(year / 100) + Math.floor(year / 400);
    return Math.floor((day + TSDaysInPreviousMonth[month] + 365 * year + leaps - exports.TSDaysFrom00000229To20010101) * exports.TSDay) + hours * exports.TSHour + minutes * exports.TSMinute + seconds;
}
exports.$timestamp = $timestamp;
function $secondFromTimestamp(ts) { return (((ts | 0) + exports.TSSecsFrom00010101To20010101) % exports.TSMinute); }
exports.$secondFromTimestamp = $secondFromTimestamp;
function $minuteFromTimestamp(ts) { return (0, commons_1.$div)(((ts | 0) + exports.TSSecsFrom00010101To20010101) % exports.TSHour, exports.TSMinute); }
exports.$minuteFromTimestamp = $minuteFromTimestamp;
function $hourFromTimestamp(ts) { return (0, commons_1.$div)(((ts | 0) + exports.TSSecsFrom00010101To20010101) % exports.TSDay, exports.TSHour); }
exports.$hourFromTimestamp = $hourFromTimestamp;
function $timestampWithoutTime(ts) {
    ts = ts | 0;
    const overhead = (ts + exports.TSSecsFrom00010101To20010101) % exports.TSDay;
    return ts - overhead;
}
exports.$timestampWithoutTime = $timestampWithoutTime;
function $dayOfYear(ts) {
    let c = (0, tsdatecomp_1.$components)(ts);
    return Math.floor((ts - $timestamp(c.year, 1, 1)) / exports.TSDay) + 1;
}
exports.$dayOfYear = $dayOfYear;
function $dayOfWeekFromTimestamp(ts, offset = 0) {
    return (Math.floor(ts / exports.TSDay) + exports.TSDaysFrom00010101To20010101 + 8 - offset) % 7;
}
exports.$dayOfWeekFromTimestamp = $dayOfWeekFromTimestamp;
function $weekOfYear(ts, offset = 0) {
    let c = (0, tsdatecomp_1.$components)(ts);
    offset %= 7;
    let ref = _yearReference(c.year, offset);
    if (ts < ref) {
        ref = _yearReference(c.year - 1, offset);
    }
    let week = Math.floor((ts - ref) / exports.TSWeek) + 1;
    if (week === 53) {
        ref += 52 * exports.TSWeek;
        c = (0, tsdatecomp_1.$components)(ref);
        if (c.day >= 29) {
            week = 1;
        }
    }
    return week;
}
exports.$weekOfYear = $weekOfYear;
/***************************************************************************************************************
 * PRIVATE FUNCTIONS AND CONSTANTS
 ***************************************************************************************************************/
const TSDaysInMonth = [0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
const TSDaysInPreviousMonth = [0, 0, 0, 0, 31, 61, 92, 122, 153, 184, 214, 245, 275, 306, 337];
function _lastDayOfMonth(year, month) {
    return (month === 2 && $isleap(year)) ? 29 : TSDaysInMonth[month];
}
function _yearReference(y, offset = 0) {
    const firstDayOfYear = $timestamp(y, 1, 1);
    let d = $dayOfWeekFromTimestamp(firstDayOfYear, offset);
    d = (d <= 3 ? -d : 7 - d);
    return firstDayOfYear + d * exports.TSDay;
}
//# sourceMappingURL=tsdate.js.map