"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TSExpectAgent = exports.TSUnaryTest = exports.TSTestGroup = exports.TSTest = exports.TSTester = void 0;
const commons_1 = require("./commons");
const compare_1 = require("./compare");
const utils_1 = require("./utils");
class TSTester {
    constructor(s) {
        this.groups = [];
        this.desc = s;
    }
    addGroup(grp) { this.addGroups([grp]); }
    addGroups(grps) {
        if (grps instanceof TSTestGroup) {
            grps = [grps];
        }
        for (let g of grps) {
            this.groups.push(g);
        }
    }
    log(format, ...args) {
        (0, utils_1.$logterm)(format, '', args);
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            // first phase, we construct all the unary test in an asynchronus function call
            this.log(`Running tests &Y&k${this.desc}&0:`);
            for (let g of this.groups) {
                yield g.prepare();
            }
            let passed = 0, failed = 0;
            // second phase, we run the tests
            for (let g of this.groups) {
                const groupok = yield g.run();
                if (groupok) {
                    passed++;
                }
                else {
                    failed++;
                }
            }
            if (passed > 0) {
                this.log(`&g${passed}&0 group${passed === 1 ? '' : 's'} of tests did &G&w  PASS  &0`);
            }
            if (failed > 0) {
                this.log(`&r${failed}&0 group${failed === 1 ? '' : 's'} of tests did &R&w  FAIL  &0`);
            }
            if (passed > 0 && !failed) {
                this.log('&G&w  ALL TESTS PASSED  &0');
            }
        });
    }
}
exports.TSTester = TSTester;
class TSTest {
    constructor(s, f) {
        this.desc = s;
        this.fn = f;
    }
    static group(s, f) {
        return new TSTestGroup(s, f);
    }
}
exports.TSTest = TSTest;
class TSTestGroup extends TSTest {
    constructor(s, f) {
        super(s, f);
        this._unaries = [];
    }
    unary(s, f) {
        this._unaries.push(new TSUnaryTest(this, s, f));
    }
    prepare() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fn(this);
        });
    }
    log(format, ...args) {
        (0, utils_1.$logterm)('  ' + format, '', args);
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            let failed = 0;
            this.log(`- Running group tests &C&k ${this.desc} &0`);
            for (let u of this._unaries) {
                const unaryok = yield u.run();
                if (!unaryok) {
                    failed++;
                }
            }
            this.log(`  ... group test ${this.desc} ${failed == 0 ? '&G&w  PASSED  ' : '&R&w  FAILED  '}&0`);
            return failed === 0;
        });
    }
}
exports.TSTestGroup = TSTestGroup;
class TSUnaryTest extends TSTest {
    constructor(g, s, f) {
        super(s, f);
        this._failed = 0;
        this._passed = 0;
        this._group = g;
    }
    log(format, ...args) {
        (0, utils_1.$logterm)('    ' + format, '', args);
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            this._passed = 0;
            this._failed = 0;
            this.log(`... running tests &B&w ${this.desc} &0`);
            yield this.fn(this);
            if (this._passed > 0) {
                this.log(`    &g${this._passed}&0 passed`);
            }
            if (this._failed > 0) {
                this.log(`    &r${this._failed}&0 failed`);
            }
            return this._failed === 0;
        });
    }
    expect(v) {
        return new TSExpectAgent(this, v);
    }
    fail() { this._failed++; }
    pass() { this._passed++; }
}
exports.TSUnaryTest = TSUnaryTest;
class TSExpectAgent {
    constructor(step, value) {
        this._step = step;
        this._value = value;
    }
    toBe(aValue) { var _a, _b; if (!(0, compare_1.$equal)(aValue, this._value)) {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.fail();
    }
    else {
        (_b = this._step) === null || _b === void 0 ? void 0 : _b.pass();
    } }
    notToBe(aValue) { var _a, _b; if ((0, compare_1.$equal)(aValue, this._value)) {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.fail();
    }
    else {
        (_b = this._step) === null || _b === void 0 ? void 0 : _b.pass();
    } }
    toBeTruthy() { var _a, _b; if (!this._value) {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.fail();
    }
    else {
        (_b = this._step) === null || _b === void 0 ? void 0 : _b.pass();
    } }
    toBeFalsy() { var _a, _b; if (this._value) {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.fail();
    }
    else {
        (_b = this._step) === null || _b === void 0 ? void 0 : _b.pass();
    } }
    toBeUndefined() { var _a, _b; if (this._value !== undefined) {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.fail();
    }
    else {
        (_b = this._step) === null || _b === void 0 ? void 0 : _b.pass();
    } }
    toBeDefined() { var _a, _b; if (this._value === undefined) {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.fail();
    }
    else {
        (_b = this._step) === null || _b === void 0 ? void 0 : _b.pass();
    } }
    toBeNull() { var _a, _b; if (this._value !== null) {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.fail();
    }
    else {
        (_b = this._step) === null || _b === void 0 ? void 0 : _b.pass();
    } }
    toBeNotNull() { var _a, _b; if (this._value === null) {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.fail();
    }
    else {
        (_b = this._step) === null || _b === void 0 ? void 0 : _b.pass();
    } }
    toBeOK() { var _a, _b; if (!(0, commons_1.$ok)(this._value)) {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.fail();
    }
    else {
        (_b = this._step) === null || _b === void 0 ? void 0 : _b.pass();
    } }
    toBeNotOK() { var _a, _b; if ((0, commons_1.$ok)(this._value)) {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.fail();
    }
    else {
        (_b = this._step) === null || _b === void 0 ? void 0 : _b.pass();
    } }
    toBeNaN() { var _a, _b; if (!isNaN(this._value)) {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.fail();
    }
    else {
        (_b = this._step) === null || _b === void 0 ? void 0 : _b.pass();
    } }
    toBeNumber() { var _a, _b; if (!(0, commons_1.$isnumber)(this._value)) {
        (_a = this._step) === null || _a === void 0 ? void 0 : _a.fail();
    }
    else {
        (_b = this._step) === null || _b === void 0 ? void 0 : _b.pass();
    } }
}
exports.TSExpectAgent = TSExpectAgent;
//# sourceMappingURL=tstester.js.map